/*

Licensed under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/agpl-3.0.html

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package application

import (
	"github.com/presslabs/controller-util/syncer"
	settings "k8s.libre.sh/application/settings"
	interfaces "k8s.libre.sh/interfaces"
	meta "k8s.libre.sh/meta"
	objects "k8s.libre.sh/objects"
)

type Component interface {
	meta.Instance
	GetMeta() meta.Instance
	SetDefaults()
	Init()
	GetObjects() map[int]objects.Object
	HasDependencies() bool
	GetDependencies() settings.Config
}

func NewSyncer(i Component, r interfaces.Reconcile, owner interfaces.Object) (syncers []syncer.Interface) {
	mutators := i.GetObjects()

	for _, m := range mutators {
		syncers = append(syncers, objects.NewObjectSyncer(m, owner, r))
	}

	return syncers
}

func InitComponent(i Instance, c Component) {

	component := c.GetComponent()

	meta.SetObjectMeta(i, c)

	if len(component) > 0 {
		c.SetComponent(component)
	}

	for _, o := range c.GetObjects() {
		meta.SetObjectMeta(c, o)
	}

}
