/*

Licensed under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/agpl-3.0.html

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package application

import (
	"context"

	"github.com/presslabs/controller-util/syncer"

	settings "k8s.libre.sh/application/settings"
	"k8s.libre.sh/interfaces"
	"k8s.libre.sh/meta"
	"k8s.libre.sh/objects"
)

type Instance interface {
	meta.Instance
	GetComponents() map[int]Component
	GetOwner() interfaces.Object
	GetSettings() settings.Settings
	//	GetSettings() map[int]settings.Settings
	SetDefaults()
}

func Init(i Instance) {

	i.SetDefaults()

	for _, c := range i.GetComponents() {
		c.Init()

		// TODO TOFIX
		component := c.GetComponent()

		meta.SetObjectMeta(i, c)

		if len(component) > 0 {
			c.SetComponent(component)
		}

		for _, o := range c.GetObjects() {
			meta.SetObjectMeta(c, o)
		}

		c.SetDefaults()

	}
}

func NewSyncers(app Instance, r interfaces.Reconcile, owner interfaces.Object) (syncers []syncer.Interface, err error) {

	////////////////////
	//// Settings //////
	///////////////////

	// TODO TO FIX SHOULD BE RUN IN INITIALIZED FUNCTION
	// INITIALISE VALUES FROM EXTERNAL RESOURCES, RANDOM & TEMPLATES
	//	sett := app.GetSettings().GetConfig().(*components.Settings)
	sett := app.GetSettings().(*settings.Component)

	err = sett.Init(r.GetClient())

	if err != nil {
		return syncers, err
	}

	// Object Syncers

	for _, obj := range sett.GetObjects() {
		//	meta.SetObjectMeta(sett.CommonMeta, obj)
		s := objects.NewObjectSyncer(obj, owner, r)

		if err := syncer.Sync(context.TODO(), s, r.GetRecorder()); err != nil {
			return syncers, err
		}

	}

	/////////////////////
	//// Components ////
	///////////////////
	components := app.GetComponents()

	// TODO TO FIX SHOULD BE RUN IN INITIALIZED FUNCTION
	// INITIALIZE
	for _, c := range components {
		if c.HasDependencies() {

			deps := c.GetDependencies()
			deps = settings.MergeSettings(sett, deps)

		}
		// GET SYNCERS
		mutators := c.GetObjects()
		for _, m := range mutators {
			syncers = append(syncers, objects.NewObjectSyncer(m, owner, r))
		}
	}
	return syncers, nil

}
